// エンコーディング：UTF-8
// PRNG.cs includes six files: RandomBase.cs, MersenneTwister.cs, RanrotB.cs, SFMT.cs, Well.cs, and Xorshift.cs
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Lotosummary.Tabulate
{

/*
 * Copyright (C) Rei HOBARA 2007
 * 
 * Name:
 *     RandomBase.cs
 * Class:
 *     Rei.Random.RandomBase
 * Purpose:
 *     A base class for random number generator.
 * Remark:
 * History:
 *     2007/10/6 initial release.
 * 
 */

// using System;

namespace Rei.Random {

    /// <summary>
    /// 各種擬似乱数ジェネレーター用基底クラス。
    /// 派生クラスはNextUInt32を実装する必要があります。
    /// </summary>
    public abstract class RandomBase {

        /// <summary>
        /// 派生クラスで符号なし32bitの擬似乱数を生成する必要があります。
        /// </summary>
        public abstract UInt32 NextUInt32();

        /// <summary>
        /// 符号あり32bitの擬似乱数を取得します。
        /// </summary>
        public virtual Int32 NextInt32() {
            return (Int32)NextUInt32();
        }

        /// <summary>
        /// 符号なし64bitの擬似乱数を取得します。
        /// </summary>
        public virtual UInt64 NextUInt64() {
            return ((UInt64)NextUInt32() << 32) | NextUInt32();
        }

        /// <summary>
        /// 符号あり64bitの擬似乱数を取得します。
        /// </summary>
        public virtual Int64 NextInt64() {
            return ((Int64)NextUInt32() << 32) | NextUInt32();
        }

        /// <summary>
        /// 擬似乱数列を生成し、バイト配列に順に格納します。
        /// </summary>
        public virtual void NextBytes( byte[] buffer ) {
            int i = 0;
            UInt32 r;
            while (i + 4 <= buffer.Length) {
                r = NextUInt32();
                buffer[i++] = (byte)r;
                buffer[i++] = (byte)(r >> 8);
                buffer[i++] = (byte)(r >> 16);
                buffer[i++] = (byte)(r >> 24);
            }
            if (i >= buffer.Length) return;
            r = NextUInt32();
            buffer[i++] = (byte)r;
            if (i >= buffer.Length) return;
            buffer[i++] = (byte)(r >> 8);
            if (i >= buffer.Length) return;
            buffer[i++] = (byte)(r >> 16);
        }

        /// <summary>
        /// [0,1)の擬似乱数を取得します。
        /// [0,1)を2^53個に均等にわけ、そのうち一つを返します。
        /// NextUInt32を2回呼び出します。
        /// </summary>
        public virtual double NextDouble() {
            UInt32 r1, r2;
            r1 = NextUInt32();
            r2 = NextUInt32();
            return (r1 * (double)(2 << 11) + r2) / (double)(2 << 53);
        }

    }

} // End of RandomBase.cs

/*
 * Copyright (C) Rei HOBARA 2007
 * 
 * Name:
 *     MersenneTwister.cs
 * Class:
 *     Rei.Random.MersenneTwister
 * Purpose:
 *     A random number generator using Mersenne Twister.
 * Remark:
 *     This code is C# implementation of Mersenne Twister.
 *     Mersenne Twister was introduced by Takuji Nishimura and Makoto Matsumoto.
 *     See http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/mt.html for detail of Mersenne Twister.
 * History:
 *     2007/10/6 initial release.
 * 
 */

// using System;

namespace Rei.Random {

    /// <summary>
    /// MersenneTwisterの擬似乱数ジェネレータークラス。
    /// </summary>
    public class MersenneTwister : RandomBase {

        #region Field

        /// <summary>
        /// 内部状態ベクトル総数
        /// </summary>
        protected const int N = 624;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const int M = 397;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const uint MATRIX_A = 0x9908b0dfU;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const uint UPPER_MASK = 0x80000000U;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const uint LOWER_MASK = 0x7fffffffU;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const uint TEMPER1 = 0x9d2c5680U;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const uint TEMPER2 = 0xefc60000U;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const int TEMPER3 = 11;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const int TEMPER4 = 7;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const int TEMPER5 = 15;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected const int TEMPER6 = 18;

        /// <summary>
        /// 内部状態ベクトル。
        /// </summary>
        protected UInt32[] mt;
        /// <summary>
        /// 内部状態ベクトルのうち、次に乱数として使用するインデックス。
        /// </summary>
        protected int mti;
        private UInt32[] mag01;

        #endregion

        /// <summary>
        /// 現在時刻を種とした、MersenneTwister擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public MersenneTwister() : this(Environment.TickCount) { }

        /// <summary>
        /// seedを種とした、MersenneTwister擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public MersenneTwister( int seed ) {
            mt = new UInt32[N];
            mti = N + 1;
            mag01 = new UInt32[] { 0x0U, MATRIX_A };
            //内部状態配列初期化
            mt[0] = (UInt32)seed;
            for (int i = 1; i < N; i++)
                mt[i] = (UInt32)(1812433253 * (mt[i - 1] ^ (mt[i - 1] >> 30)) + i);
        }

        /// <summary>
        /// 符号なし32bitの擬似乱数を取得します。
        /// </summary>
        public override uint NextUInt32() {
            UInt32 y;
            if (mti >= N) { gen_rand_all(); mti = 0; }
            y = mt[mti++];
            y ^= (y >> TEMPER3);
            y ^= (y << TEMPER4) & TEMPER1;
            y ^= (y << TEMPER5) & TEMPER2;
            y ^= (y >> TEMPER6);
            return y;
        }

        /// <summary>
        /// 内部状態ベクトルを更新します。
        /// </summary>
        protected void gen_rand_all() {
            int kk = 1;
            UInt32 y;
            UInt32 p;
            y = mt[0] & UPPER_MASK;
            do {
                p = mt[kk];
                mt[kk - 1] = mt[kk + (M - 1)] ^ ((y | (p & LOWER_MASK)) >> 1) ^ mag01[p & 1];
                y = p & UPPER_MASK;
            } while (++kk < N - M + 1);
            do {
                p = mt[kk];
                mt[kk - 1] = mt[kk + (M - N - 1)] ^ ((y | (p & LOWER_MASK)) >> 1) ^ mag01[p & 1];
                y = p & UPPER_MASK;
            } while (++kk < N);
            p = mt[0];
            mt[N - 1] = mt[M - 1] ^ ((y | (p & LOWER_MASK)) >> 1) ^ mag01[p & 1];
        }

    }
} // End of MersenneTwister.cs

/*
 * Copyright (C) Rei HOBARA 2007
 * 
 * Name:
 *     Well.cs
 * Class:
 *     Rei.Random.RanrotB
 * Purpose:
 *     A random number generator using RanrotB.
 * Remark:
 *     This code is C# implementation of RanrotB.
 *     Well was introduced by Agner Fog.
 *     See http://www.agner.org/random/theory/chaosran.pdf for detail of RanrotB.
 * History:
 *     2007/10/6 initial release.
 * 
 */

// using System;

namespace Rei.Random {

    /// <summary>
    /// Ranrotの擬似乱数ジェネレータークラス。
    /// </summary>
    public class RanrotB : RandomBase {

        #region Field

        /// <summary>
        /// 内部状態ベクトルの個数。
        /// </summary>
        protected const int KK = 17;
        /// <summary>
        /// RanrotBのパラメーターの一つ。
        /// </summary>
        protected const int JJ = 10;
        /// <summary>
        /// RanrotBのパラメーターの一つ。
        /// </summary>
        protected const int R1 = 13;
        /// <summary>
        /// RanrotBのパラメーターの一つ。
        /// </summary>
        protected const int R2 = 9;
        /// <summary>
        /// 内部状態ベクトル。
        /// </summary>
        protected UInt32[] randbuffer;
        /// <summary>
        /// リングバッファのインデックス。
        /// </summary>
        protected int p1, p2;

        #endregion

        /// <summary>
        /// 現在時刻を種とした、Well擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public RanrotB() : this(Environment.TickCount) { }

        /// <summary>
        /// seedを種とした、Well擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public RanrotB( int seed ) {
            UInt32 s = (UInt32)seed;
            randbuffer = new UInt32[KK];
            for (int i = 0; i < KK; i++)
                randbuffer[i] = s = s * 2891336453 + 1;
            p1 = 0; p2 = JJ;
            for (int i = 0; i < 9; i++) NextUInt32();
        }

        /// <summary>
        /// 符号なし32bitの擬似乱数を取得します。
        /// </summary>
        public override uint NextUInt32() {
            UInt32 x;
            x = randbuffer[p1] = ((randbuffer[p2] << R1) | (randbuffer[p2] >> (32 - R1))) + ((randbuffer[p1] << R2) | (randbuffer[p1] >> (32 - R2)));
            if (--p1 < 0) p1 = KK - 1;
            if (--p2 < 0) p2 = KK - 1;
            return x;
        }

    }

} // End of RanrotB.cs

/*
 * Copyright (C) Rei HOBARA 2007
 * 
 * Name:
 *     SFMT.cs
 * Class:
 *     Rei.Random.SFMT
 *     Rei.Random.MTPeriodType
 * Purpose:
 *     A random number generator using SIMD-oriented Fast Mersenne Twister(SFMT).
 * Remark:
 *     This code is C# implementation of SFMT.
 *     SFMT was introduced by Mutsuo Saito and Makoto Matsumoto.
 *     See http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/SFMT/index.html for detail of SFMT.
 * History:
 *     2007/10/6 initial release.
 * 
 */

// using System;

namespace Rei.Random {

    /// <summary>
    /// SFMTの擬似乱数ジェネレータークラス。
    /// </summary>
    public class SFMT : RandomBase {

        #region Fields

        /// <summary>
        /// 周期を表す指数。
        /// </summary>
        protected int MEXP;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected int POS1;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected int SL1;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected int SL2;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected int SR1;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected int SR2;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected UInt32 MSK1;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected UInt32 MSK2;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected UInt32 MSK3;
        /// <summary>
        /// MTを決定するパラメーターの一つ。
        /// </summary>
        protected UInt32 MSK4;
        /// <summary>
        /// MTの周期を保証するための確認に用いるパラメーターの一つ。
        /// </summary>
        protected UInt32 PARITY1;
        /// <summary>
        /// MTの周期を保証するための確認に用いるパラメーターの一つ。
        /// </summary>
        protected UInt32 PARITY2;
        /// <summary>
        /// MTの周期を保証するための確認に用いるパラメーターの一つ。
        /// </summary>
        protected UInt32 PARITY3;
        /// <summary>
        /// MTの周期を保証するための確認に用いるパラメーターの一つ。
        /// </summary>
        protected UInt32 PARITY4;

        /// <summary>
        /// 各要素を128bitとしたときの内部状態ベクトルの個数。
        /// </summary>
        protected int N;
        /// <summary>
        /// 各要素を32bitとしたときの内部状態ベクトルの個数。
        /// </summary>
        protected int N32;
        /// <summary>
        /// 計算の高速化用。
        /// </summary>
        protected int SL2_x8;
        /// <summary>
        /// 計算の高速化用。
        /// </summary>
        protected int SR2_x8;
        /// <summary>
        /// 計算の高速化用。
        /// </summary>
        protected int SL2_ix8;
        /// <summary>
        /// 計算の高速化用。
        /// </summary>
        protected int SR2_ix8;

        /// <summary>
        /// 内部状態ベクトル。
        /// </summary>
        protected UInt32[] sfmt;
        /// <summary>
        /// 内部状態ベクトルのうち、次に乱数として使用するインデックス。
        /// </summary>
        protected int idx;

        #endregion

        /// <summary>
        /// 現在時刻を種とした、(2^19937-1)周期のSFMT擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public SFMT() : this(Environment.TickCount, 19937) { }

        /// <summary>
        /// seedを種とした、(2^19937-1)周期の擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public SFMT( int seed ) : this(seed, 19937) { }

        /// <summary>
        /// seedを種とした、periodで表される周期の擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public SFMT( int seed, MTPeriodType period ) : this(seed, (int)period) { }

        /// <summary>
        /// seedを種とした、(2^mexp-1)周期の擬似乱数ジェネレーターを初期化します。
        /// mexpは607,1279,2281,4253,11213,19937,44497,86243,132049,216091のいずれかである必要があります。
        /// </summary>
        public SFMT( int seed, int mexp ) {
            this.MEXP = mexp;
            if (mexp == 607) {
                POS1 = 2;
                SL1 = 15;
                SL2 = 3;
                SR1 = 13;
                SR2 = 3;
                MSK1 = 0xfdff37ffU;
                MSK2 = 0xef7f3f7dU;
                MSK3 = 0xff777b7dU;
                MSK4 = 0x7ff7fb2fU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0x00000000U;
                PARITY4 = 0x5986f054U;
            } else if (mexp == 1279) {
                POS1 = 7;
                SL1 = 14;
                SL2 = 3;
                SR1 = 5;
                SR2 = 1;
                MSK1 = 0xf7fefffdU;
                MSK2 = 0x7fefcfffU;
                MSK3 = 0xaff3ef3fU;
                MSK4 = 0xb5ffff7fU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0x00000000U;
                PARITY4 = 0x20000000U;
            } else if (mexp == 2281) {
                POS1 = 12;
                SL1 = 19;
                SL2 = 1;
                SR1 = 5;
                SR2 = 1;
                MSK1 = 0xbff7ffbfU;
                MSK2 = 0xfdfffffeU;
                MSK3 = 0xf7ffef7fU;
                MSK4 = 0xf2f7cbbfU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0x00000000U;
                PARITY4 = 0x41dfa600U;
            } else if (mexp == 4253) {
                POS1 = 17;
                SL1 = 20;
                SL2 = 1;
                SR1 = 7;
                SR2 = 1;
                MSK1 = 0x9f7bffffU;
                MSK2 = 0x9fffff5fU;
                MSK3 = 0x3efffffbU;
                MSK4 = 0xfffff7bbU;
                PARITY1 = 0xa8000001U;
                PARITY2 = 0xaf5390a3U;
                PARITY3 = 0xb740b3f8U;
                PARITY4 = 0x6c11486dU;
            } else if (mexp == 11213) {
                POS1 = 68;
                SL1 = 14;
                SL2 = 3;
                SR1 = 7;
                SR2 = 3;
                MSK1 = 0xeffff7fbU;
                MSK2 = 0xffffffefU;
                MSK3 = 0xdfdfbfffU;
                MSK4 = 0x7fffdbfdU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0xe8148000U;
                PARITY4 = 0xd0c7afa3U;
            } else if (mexp == 19937) {
                POS1 = 122;
                SL1 = 18;
                SL2 = 1;
                SR1 = 11;
                SR2 = 1;
                MSK1 = 0xdfffffefU;
                MSK2 = 0xddfecb7fU;
                MSK3 = 0xbffaffffU;
                MSK4 = 0xbffffff6U;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0x00000000U;
                PARITY4 = 0x13c9e684U;
                PARITY4 = 0x20000000U;
            } else if (mexp == 44497) {
                POS1 = 330;
                SL1 = 5;
                SL2 = 3;
                SR1 = 9;
                SR2 = 3;
                MSK1 = 0xeffffffbU;
                MSK2 = 0xdfbebfffU;
                MSK3 = 0xbfbf7befU;
                MSK4 = 0x9ffd7bffU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0xa3ac4000U;
                PARITY4 = 0xecc1327aU;
            } else if (mexp == 86243) {
                POS1 = 366;
                SL1 = 6;
                SL2 = 7;
                SR1 = 19;
                SR2 = 1;
                MSK1 = 0xfdbffbffU;
                MSK2 = 0xbff7ff3fU;
                MSK3 = 0xfd77efffU;
                MSK4 = 0xbf9ff3ffU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0x00000000U;
                PARITY4 = 0xe9528d85U;
            } else if (mexp == 132049) {
                POS1 = 110;
                SL1 = 19;
                SL2 = 1;
                SR1 = 21;
                SR2 = 1;
                MSK1 = 0xffffbb5fU;
                MSK2 = 0xfb6ebf95U;
                MSK3 = 0xfffefffaU;
                MSK4 = 0xcff77fffU;
                PARITY1 = 0x00000001U;
                PARITY2 = 0x00000000U;
                PARITY3 = 0xcb520000U;
                PARITY4 = 0xc7e91c7dU;
            } else if (mexp == 216091) {
                POS1 = 627;
                SL1 = 11;
                SL2 = 3;
                SR1 = 10;
                SR2 = 1;
                MSK1 = 0xbff7bff7U;
                MSK2 = 0xbfffffffU;
                MSK3 = 0xbffffa7fU;
                MSK4 = 0xffddfbfbU;
                PARITY1 = 0xf8000001U;
                PARITY2 = 0x89e80709U;
                PARITY3 = 0x3bd2b64bU;
                PARITY4 = 0x0c64b1e4U;
            } else {
                throw new ArgumentException();
            }
            init_gen_rand(seed);
        }

        /// <summary>
        /// 符号なし32bitの擬似乱数を取得します。
        /// </summary>
        public override UInt32 NextUInt32() {
            if (idx >= N32) {
                gen_rand_all();
                idx = 0;
            }
            return sfmt[idx++];
        }

        /// <summary>
        /// ジェネレーターを初期化します。
        /// </summary>
        /// <param name="seed"></param>
        protected void init_gen_rand( int seed ) {
            int i;
            //変数初期化
            N = MEXP / 128 + 1;
            N32 = N * 4;
            SL2_x8 = SL2 * 8;
            SR2_x8 = SR2 * 8;
            SL2_ix8 = 64 - SL2 * 8;
            SR2_ix8 = 64 - SR2 * 8;
            //内部状態配列確保
            sfmt = new UInt32[N32];
            //内部状態配列初期化
            sfmt[0] = (UInt32)seed;
            for (i = 1; i < N32; i++)
                sfmt[i] = (UInt32)(1812433253 * (sfmt[i - 1] ^ (sfmt[i - 1] >> 30)) + i);
            //確認
            period_certification();
            //初期位置設定
            idx = N32;
        }

        /// <summary>
        /// 内部状態ベクトルが適切か確認し、必要であれば調節します。
        /// </summary>
        protected void period_certification() {
            UInt32[] PARITY = new UInt32[] { PARITY1, PARITY2, PARITY3, PARITY4 };
            UInt32 inner = 0;
            int i, j;
            UInt32 work;

            for (i = 0; i < 4; i++) inner ^= sfmt[i] & PARITY[i];
            for (i = 16; i > 0; i >>= 1) inner ^= inner >> i;
            inner &= 1;
            // check OK
            if (inner == 1) return;
            // check NG, and modification
            for (i = 0; i < 4; i++) {
                work = 1;
                for (j = 0; j < 32; j++) {
                    if ((work & PARITY[i]) != 0) {
                        sfmt[i] ^= work;
                        return;
                    }
                    work = work << 1;
                }
            }
        }

        /// <summary>
        /// 内部状態ベクトルを更新します。
        /// </summary>
        protected virtual void gen_rand_all() {
            if (MEXP == 19937) { gen_rand_all_19937(); return; }
            int a, b, c, d;
            UInt64 xh, xl, yh, yl;

            a = 0;
            b = POS1 * 4;
            c = (N - 2) * 4;
            d = (N - 1) * 4;
            do {
                xh = ((UInt64)sfmt[a + 3] << 32) | sfmt[a + 2];
                xl = ((UInt64)sfmt[a + 1] << 32) | sfmt[a + 0];
                yh = xh << (SL2_x8) | xl >> (SL2_ix8);
                yl = xl << (SL2_x8);
                xh = ((UInt64)sfmt[c + 3] << 32) | sfmt[c + 2];
                xl = ((UInt64)sfmt[c + 1] << 32) | sfmt[c + 0];
                yh ^= xh >> (SR2_x8);
                yl ^= xl >> (SR2_x8) | xh << (SR2_ix8);

                sfmt[a + 3] = sfmt[a + 3] ^ ((sfmt[b + 3] >> SR1) & MSK4) ^ (sfmt[d + 3] << SL1) ^ ((UInt32)(yh >> 32));
                sfmt[a + 2] = sfmt[a + 2] ^ ((sfmt[b + 2] >> SR1) & MSK3) ^ (sfmt[d + 2] << SL1) ^ ((UInt32)yh);
                sfmt[a + 1] = sfmt[a + 1] ^ ((sfmt[b + 1] >> SR1) & MSK2) ^ (sfmt[d + 1] << SL1) ^ ((UInt32)(yl >> 32));
                sfmt[a + 0] = sfmt[a + 0] ^ ((sfmt[b + 0] >> SR1) & MSK1) ^ (sfmt[d + 0] << SL1) ^ ((UInt32)yl);

                c = d; d = a; a += 4; b += 4;
                if (b >= N32) b = 0;
            } while (a < N32);
        }

        /// <summary>
        /// gen_rand_allの(2^19937-1)周期用。
        /// </summary>
        private void gen_rand_all_19937() {
            int a, b, c, d;
            UInt32[] p = this.sfmt;

            const int cMEXP = 19937;
            const int cPOS1 = 122;
            const uint cMSK1 = 0xdfffffefU;
            const uint cMSK2 = 0xddfecb7fU;
            const uint cMSK3 = 0xbffaffffU;
            const uint cMSK4 = 0xbffffff6U;
            const int cSL1 = 18;
            const int cSR1 = 11;
            const int cN = cMEXP / 128 + 1;
            const int cN32 = cN * 4;

            a = 0;
            b = cPOS1 * 4;
            c = (cN - 2) * 4;
            d = (cN - 1) * 4;
            do {
                p[a + 3] = p[a + 3] ^ (p[a + 3] << 8) ^ (p[a + 2] >> 24) ^ (p[c + 3] >> 8) ^ ((p[b + 3] >> cSR1) & cMSK4) ^ (p[d + 3] << cSL1);
                p[a + 2] = p[a + 2] ^ (p[a + 2] << 8) ^ (p[a + 1] >> 24) ^ (p[c + 3] << 24) ^ (p[c + 2] >> 8) ^ ((p[b + 2] >> cSR1) & cMSK3) ^ (p[d + 2] << cSL1);
                p[a + 1] = p[a + 1] ^ (p[a + 1] << 8) ^ (p[a + 0] >> 24) ^ (p[c + 2] << 24) ^ (p[c + 1] >> 8) ^ ((p[b + 1] >> cSR1) & cMSK2) ^ (p[d + 1] << cSL1);
                p[a + 0] = p[a + 0] ^ (p[a + 0] << 8) ^ (p[c + 1] << 24) ^ (p[c + 0] >> 8) ^ ((p[b + 0] >> cSR1) & cMSK1) ^ (p[d + 0] << cSL1);
                c = d; d = a; a += 4; b += 4;
                if (b >= cN32) b = 0;
            } while (a < cN32);
        }

    }

    /// <summary>
    /// メルセンヌツイスターで用いる周期をあらわす列挙型。
    /// </summary>
    public enum MTPeriodType {
        /// <summary>
        /// 2^607-1周期のMT。
        /// </summary>
        MT607 = 607,
        /// <summary>
        /// 2^1279-1周期のMT。
        /// </summary>
        MT1279 = 1279,
        /// <summary>
        /// 2^2281-1周期のMT。
        /// </summary>
        MT2281 = 2281,
        /// <summary>
        /// 2^4253-1周期のMT。
        /// </summary>
        MT4253 = 4253,
        /// <summary>
        /// 2^11213-1周期のMT。
        /// </summary>
        MT11213 = 11213,
        /// <summary>
        /// 2^19937-1周期のMT。
        /// </summary>
        MT19937 = 19937,
        /// <summary>
        /// 2^44497-1周期のMT。
        /// </summary>
        MT44497 = 44497,
        /// <summary>
        /// 2^86243-1周期のMT。
        /// </summary>
        MT86243 = 86243,
        /// <summary>
        /// 2^132049-1周期のMT。
        /// </summary>
        MT132049 = 132049,
        /// <summary>
        /// 2^216091-1周期のMT。
        /// </summary>
        MT216091 = 216091
    }

} // End of SFMT.cs

/*
 * Copyright (C) Rei HOBARA 2007
 * 
 * Name:
 *     Xorshift.cs
 * Class:
 *     Rei.Random.Xorshift
 * Purpose:
 *     A random number generator using Xorshift.
 * Remark:
 *     This code is C# implementation of Xorshift.
 *     Xorshift was introduced by George Marsaglia.
 *     See http://www.jstatsoft.org/v08/i14/paper for detail of Xorshift.
 * History:
 *     2007/10/6 initial release.
 * 
 */

// using System;

namespace Rei.Random {

    /// <summary>
    /// Xorshiftの擬似乱数ジェネレータークラス。
    /// </summary>
    public class Xorshift : RandomBase {

        /// <summary>
        /// 内部状態ベクトル。
        /// </summary>
        protected UInt32 x, y, z, w;

        /// <summary>
        /// 現在時刻を種とした、Xorshift擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public Xorshift() : this(Environment.TickCount) { }

        //static unsigned long x=123456789,y=362436069,z=521288629,w=88675123;

        /// <summary>
        /// seedを種とした、Xorshift擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public Xorshift( int seed ) : this((UInt32)seed, 362436069, 521288629, 88675123) { }

        /// <summary>
        /// seedを種とした、Xorshift擬似乱数ジェネレーターを初期化します。
        /// George Marsagliaによるオリジナルはseed1=123456789,seed2=362436069,seed3=521288629,seed4=88675123を用いています。
        /// </summary>
        /// <param name="seed1"></param>
        /// <param name="seed2"></param>
        /// <param name="seed3"></param>
        /// <param name="seed4"></param>
        public Xorshift( UInt32 seed1, UInt32 seed2, UInt32 seed3, UInt32 seed4 ) {
            x = seed1; y = seed2; z = seed3; w = seed4;
        }

        /// <summary>
        /// 符号なし32bitの擬似乱数を取得します。
        /// </summary>
        public override uint NextUInt32() {
            UInt32 t;
            t = (x ^ (x << 11));
            x = y; y = z; z = w;
            return (w = (w ^ (w >> 19)) ^ (t ^ (t >> 8)));
        }
    }

} // End of Xorshift.cs

/*
 * Copyright (C) Rei HOBARA 2007
 * 
 * Name:
 *     Well.cs
 * Class:
 *     Rei.Random.Well
 * Purpose:
 *     A random number generator using Well.
 * Remark:
 *     This code is C# implementation of Well.
 *     Well was introduced by Francois panneton, Pierre L'ecuyer and Makoto Matsumoto.
 *     See http://www.iro.umontreal.ca/~lecuyer/myftp/papers/wellrng.pdf for detail of Well.
 * History:
 *     2007/10/6 initial release.
 * 
 */

// using System;

namespace Rei.Random {

    /// <summary>
    /// Wellの擬似乱数ジェネレータークラス。
    /// </summary>
    public class Well : RandomBase {

        #region Field

        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const int W = 32;
        /// <summary>
        /// 内部状態ベクトルの個数。
        /// </summary>
        protected const int R = 624;
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const int P = 31;
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const uint MASKU = (0xffffffffU >> (W - P));
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const uint MASKL = (~MASKU);
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const int M1 = 70;
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const int M2 = 179;
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const int M3 = 449;
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const uint TEMPERB = 0x12345678U;
        /// <summary>
        /// Wellのパラメーターの一つ。
        /// </summary>
        protected const uint TEMPERC = 0x87654321U;

        /// <summary>
        /// 内部状態ベクトル。
        /// </summary>
        protected UInt32[] state;
        /// <summary>
        /// 内部状態ベクトルのうち、次に乱数として使用するインデックス。
        /// </summary>
        protected int state_i;

        #endregion

        /// <summary>
        /// 現在時刻を種とした、Well擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public Well() : this(Environment.TickCount) { }

        /// <summary>
        /// seedを種とした、Well擬似乱数ジェネレーターを初期化します。
        /// </summary>
        public Well( int seed ) {
            state = new UInt32[R];
            state_i = 0;
            //内部状態配列初期化
            //スタンダードな初期化法が見つからないのでとりあえずMTと同じ初期化にしておく。
            state[0] = (UInt32)seed;
            for (int i = 1; i < R; i++)
                state[i] = (UInt32)(1812433253 * (state[i - 1] ^ (state[i - 1] >> 30)) + i);
        }

        /// <summary>
        /// 符号なし32bitの擬似乱数を取得します。
        /// </summary>
        public override uint NextUInt32() {
            uint z1, z2;
            int i1, i2, i3, i4, i5;
            if (state_i < 2) {
                if (state_i < 1) i2 = state_i - 1 + R; else i2 = state_i - 1;
                i1 = state_i - 2 + R;
            } else {
                i1 = state_i - 2;
                i2 = state_i - 1;
            }
            if (state_i + M3 >= R) {
                if (state_i + M2 >= R) {
                    if (state_i + M1 >= R) i3 = state_i + M1 - R; else i3 = state_i + M1;
                    i4 = state_i + M2 - R;
                } else {
                    i3 = state_i + M1;
                    i4 = state_i + M2;
                }
                i5 = state_i + M3 - R;
            } else {
                i3 = state_i + M1;
                i4 = state_i + M2;
                i5 = state_i + M3;
            }
            z1 = (state[state_i] ^ (state[state_i] << 25)) ^ (state[i3] ^ (state[i3] >> 27));
            z2 = (state[i4] >> 9) ^ (state[i5] ^ (state[i5] >> 1));
            state[state_i] = z1 ^ z2;
            z1 = (state[i2] & MASKL) ^ (state[i1] & MASKU) ^ (z1 << 9) ^ (z2 << 21) ^ (state[state_i] >> 21);
            state_i = i2;
            state[i2] = z1;
            z1 = z1 ^ ((z1 << 7) & TEMPERB);
            z1 = z1 ^ ((z1 << 15) & TEMPERC);
            return z1;
            //以下、直訳。
            //uint z0, z1, z2;
            //uint y;
            //if (state_i == 0) {
            //    z0 = (state[state_i + R - 1] & MASKL) | (state[state_i + R - 2] & MASKU);
            //    z1 = (state[state_i] ^ (state[state_i] << (25))) ^ (state[state_i + M1] ^ (state[state_i + M1] >> 27));
            //    z2 = (state[state_i + M2] >> 9) ^ (state[state_i + M3] ^ (state[state_i + M3] >> 1));
            //    state[state_i] = z1 ^ z2;
            //    state[state_i - 1 + R] = (z0) ^ (z1 ^ (z1 << (9))) ^ (z2 ^ (z2 << (21))) ^ (state[state_i] ^ (state[state_i] >> 21));
            //} else if (state_i == 1) {
            //    z0 = (state[state_i - 1] & MASKL) | (state[state_i + R - 2] & MASKU);
            //    z1 = (state[state_i] ^ (state[state_i] << (25))) ^ (state[state_i + M1] ^ (state[state_i + M1] >> 27));
            //    z2 = (state[state_i + M2] >> 9) ^ (state[state_i + M3] ^ (state[state_i + M3] >> 1));
            //    state[state_i] = z1 ^ z2;
            //    state[state_i - 1] = (z0) ^ (z1 ^ (z1 << (9))) ^ (z2 ^ (z2 << (21))) ^ (state[state_i] ^ (state[state_i] >> 21));
            //} else if (state_i + M1 >= R) {
            //    z0 = (state[state_i - 1] & MASKL) | (state[state_i - 2] & MASKU);
            //    z1 = (state[state_i] ^ (state[state_i] << (25))) ^ (state[state_i + M1 - R] ^ (state[state_i + M1 - R] >> 27));
            //    z2 = (state[state_i + M2 - R] >> 9) ^ (state[state_i + M3 - R] ^ (state[state_i + M3 - R] >> 1));
            //    state[state_i] = z1 ^ z2;
            //    state[state_i - 1] = (z0) ^ (z1 ^ (z1 << (9))) ^ (z2 ^ (z2 << (21))) ^ (state[state_i] ^ (state[state_i] >> 21));
            //} else if (state_i + M2 >= R) {
            //    z0 = (state[state_i - 1] & MASKL) | (state[state_i - 2] & MASKU);
            //    z1 = (state[state_i] ^ (state[state_i] << (25))) ^ (state[state_i + M1] ^ (state[state_i + M1] >> 27));
            //    z2 = (state[state_i + M2 - R] >> 9) ^ (state[state_i + M3 - R] ^ (state[state_i + M3 - R] >> 1));
            //    state[state_i] = z1 ^ z2;
            //    state[state_i - 1] = (z0) ^ (z1 ^ (z1 << (9))) ^ (z2 ^ (z2 << (21))) ^ (state[state_i] ^ (state[state_i] >> 21));
            //} else if (state_i + M3 >= R) {
            //    z0 = (state[state_i - 1] & MASKL) | (state[state_i - 2] & MASKU);
            //    z1 = (state[state_i] ^ (state[state_i] << (25))) ^ (state[state_i + M1] ^ (state[state_i + M1] >> 27));
            //    z2 = (state[state_i + M2] >> 9) ^ (state[state_i + M3 - R] ^ (state[state_i + M3 - R] >> 1));
            //    state[state_i] = z1 ^ z2;
            //    state[state_i - 1] = (z0) ^ (z1 ^ (z1 << (9))) ^ (z2 ^ (z2 << (21))) ^ (state[state_i] ^ (state[state_i] >> 21));
            //} else {
            //    z0 = (state[state_i - 1] & MASKL) | (state[state_i - 2] & MASKU);
            //    z1 = (state[state_i] ^ (state[state_i] << (25))) ^ (state[state_i + M1] ^ (state[state_i + M1] >> 27));
            //    z2 = (state[state_i + M2] >> 9) ^ (state[state_i + M3] ^ (state[state_i + M3] >> 1));
            //    state[state_i] = z1 ^ z2;
            //    state[state_i - 1] = (z0) ^ (z1 ^ (z1 << (9))) ^ (z2 ^ (z2 << (21))) ^ (state[state_i] ^ (state[state_i] >> 21));
            //}
            //state_i--;
            //if (state_i == -1) state_i = R - 1;
            //y = state[state_i] ^ ((state[state_i] << 7) & TEMPERB);
            //y = y ^ ((y << 15) & TEMPERC);
            //return y;
        }

    }

} // End of Well.cs

    class PRNG
    {
    }

}
